@@ -5,6 +5,7 @@ import (
 
 	"github.com/rclone/rclone/cmd"
 	"github.com/rclone/rclone/cmd/serve/dlna"
+	"github.com/rclone/rclone/cmd/serve/docker"
 	"github.com/rclone/rclone/cmd/serve/ftp"
 	"github.com/rclone/rclone/cmd/serve/http"
 	"github.com/rclone/rclone/cmd/serve/restic"
@@ -30,6 +31,9 @@ func init() {
 	if sftp.Command != nil {
 		Command.AddCommand(sftp.Command)
 	}
+	if docker.Command != nil {
+		Command.AddCommand(docker.Command)
+	}
 	cmd.Root.AddCommand(Command)
 }
 
@@ -38,7 +42,7 @@ var Command = &cobra.Command{
 	Use:   "serve <protocol> [opts] <remote>",
 	Short: `Serve a remote over a protocol.`,
 	Long: `rclone serve is used to serve a remote over a given protocol. This
-command requires the use of a subcommand to specify the protocol, eg
+command requires the use of a subcommand to specify the protocol, e.g.
 
     rclone serve http remote:
 
@@ -46,7 +50,7 @@ Each subcommand has its own options which you can see in their help.
 `,
 	RunE: func(command *cobra.Command, args []string) error {
 		if len(args) == 0 {
-			return errors.New("serve requires a protocol, eg 'rclone serve http remote:'")
+			return errors.New("serve requires a protocol, e.g. 'rclone serve http remote:'")
 		}
 		return errors.New("unknown protocol")
 	},
