@@ -2,6 +2,7 @@ package vfs
 
 import (
 	"context"
+	"fmt"
 	"os"
 	"path"
 	"sort"
@@ -45,9 +46,10 @@ type Dir struct {
 type vState byte
 
 const (
-	vOK  vState = iota // Not virtual
-	vAdd               // added file or directory
-	vDel               // removed file or directory
+	vOK      vState = iota // Not virtual
+	vAddFile               // added file
+	vAddDir                // added directory
+	vDel                   // removed file or directory
 )
 
 func newDir(vfs *VFS, f fs.Fs, parent *Dir, fsDir fs.Directory) *Dir {
@@ -73,6 +75,47 @@ func (d *Dir) String() string {
 	return d.path + "/"
 }
 
+// Dumps the directory tree to the string builder with the given indent
+func (d *Dir) dumpIndent(out *strings.Builder, indent string) {
+	if d == nil {
+		fmt.Fprintf(out, "%s<nil *Dir>\n", indent)
+		return
+	}
+	d.mu.RLock()
+	defer d.mu.RUnlock()
+	fmt.Fprintf(out, "%sPath: %s\n", indent, d.path)
+	fmt.Fprintf(out, "%sEntry: %v\n", indent, d.entry)
+	fmt.Fprintf(out, "%sRead: %v\n", indent, d.read)
+	fmt.Fprintf(out, "%s- items %d\n", indent, len(d.items))
+	// Sort?
+	for leaf, node := range d.items {
+		switch x := node.(type) {
+		case *Dir:
+			fmt.Fprintf(out, "%s  %s/ - %v\n", indent, leaf, x)
+			// check the parent is correct
+			if x.parent != d {
+				fmt.Fprintf(out, "%s  PARENT POINTER WRONG\n", indent)
+			}
+			x.dumpIndent(out, indent+"\t")
+		case *File:
+			fmt.Fprintf(out, "%s  %s - %v\n", indent, leaf, x)
+		default:
+			panic("bad dir entry")
+		}
+	}
+	fmt.Fprintf(out, "%s- virtual %d\n", indent, len(d.virtual))
+	for leaf, state := range d.virtual {
+		fmt.Fprintf(out, "%s  %s - %v\n", indent, leaf, state)
+	}
+}
+
+// Dumps a nicely formatted directory tree to a string
+func (d *Dir) dump() string {
+	var out strings.Builder
+	d.dumpIndent(&out, "")
+	return out.String()
+}
+
 // IsFile returns false for Dir - satisfies Node interface
 func (d *Dir) IsFile() bool {
 	return false
@@ -121,7 +164,7 @@ func (d *Dir) Inode() uint64 {
 	return d.inode
 }
 
-// Node returns the Node assocuated with this - satisfies Noder interface
+// Node returns the Node associated with this - satisfies Noder interface
 func (d *Dir) Node() Node {
 	return d
 }
@@ -144,6 +187,9 @@ func (d *Dir) ForgetAll() (hasVirtual bool) {
 			}
 		}
 	}
+	// Purge any unecessary virtual entries
+	d._purgeVirtual()
+
 	d.read = time.Time{}
 	// Check if this dir has virtual entries
 	if len(d.virtual) != 0 {
@@ -254,7 +300,7 @@ func (d *Dir) countActiveWriters() (writers int) {
 }
 
 // age returns the duration since the last time the directory contents
-// was read and the content is cosidered stale. age will be 0 and
+// was read and the content is considered stale. age will be 0 and
 // stale true if the last read time is empty.
 // age must be called with d.mu held.
 func (d *Dir) _age(when time.Time) (age time.Duration, stale bool) {
@@ -266,21 +312,56 @@ func (d *Dir) _age(when time.Time) (age time.Duration, stale bool) {
 	return
 }
 
+// renameTree renames the directories under this directory
+//
+// path should be the desired path
+func (d *Dir) renameTree(dirPath string) {
+	d.mu.Lock()
+	defer d.mu.Unlock()
+
+	// Make sure the path is correct for each node
+	if d.path != dirPath {
+		fs.Debugf(d.path, "Renaming to %q", dirPath)
+		d.path = dirPath
+		d.entry = fs.NewDirCopy(context.TODO(), d.entry).SetRemote(dirPath)
+	}
+
+	// Do the same to any child directories
+	for leaf, node := range d.items {
+		if dir, ok := node.(*Dir); ok {
+			dir.renameTree(path.Join(dirPath, leaf))
+		}
+	}
+}
+
 // rename should be called after the directory is renamed
 //
 // Reset the directory to new state, discarding all the objects and
 // reading everything again
 func (d *Dir) rename(newParent *Dir, fsDir fs.Directory) {
 	d.ForgetAll()
+
 	d.modTimeMu.Lock()
 	d.modTime = fsDir.ModTime(context.TODO())
 	d.modTimeMu.Unlock()
 	d.mu.Lock()
+	oldPath := d.path
 	d.parent = newParent
 	d.entry = fsDir
 	d.path = fsDir.Remote()
+	newPath := d.path
 	d.read = time.Time{}
 	d.mu.Unlock()
+
+	// Rename any remaining items in the tree that we couldn't forget
+	d.renameTree(d.path)
+
+	// Rename in the cache
+	if d.vfs.cache != nil && d.vfs.cache.DirExists(oldPath) {
+		if err := d.vfs.cache.DirRename(oldPath, newPath); err != nil {
+			fs.Infof(d, "Dir.Rename failed in Cache: %v", err)
+		}
+	}
 }
 
 // addObject adds a new object or directory to the directory
@@ -296,6 +377,10 @@ func (d *Dir) addObject(node Node) {
 	if d.virtual == nil {
 		d.virtual = make(map[string]vState)
 	}
+	vAdd := vAddFile
+	if node.IsDir() {
+		vAdd = vAddDir
+	}
 	d.virtual[leaf] = vAdd
 	fs.Debugf(d.path, "Added virtual directory entry %v: %q", vAdd, leaf)
 	d.mu.Unlock()
@@ -348,7 +433,7 @@ func (d *Dir) delObject(leaf string) {
 // DelVirtual removes an object from the directory listing
 //
 // It marks it as removed until it has confirmed the object is missing when the
-// directory entries are re-read in which case the virtual mark is removed.
+// directory entries are re-read in.
 //
 // This is used to remove directory entries after things have been deleted or
 // renamed but before we've had confirmation from the backend.
@@ -389,33 +474,153 @@ func (d *Dir) _readDirFromDirTree(dirTree dirtree.DirTree, when time.Time) error
 	return d._readDirFromEntries(dirTree[d.path], dirTree, when)
 }
 
+// Remove the virtual directory entry leaf
+func (d *Dir) _deleteVirtual(name string) {
+	virtualState, ok := d.virtual[name]
+	if !ok {
+		return
+	}
+	delete(d.virtual, name)
+	if len(d.virtual) == 0 {
+		d.virtual = nil
+	}
+	fs.Debugf(d.path, "Removed virtual directory entry %v: %q", virtualState, name)
+}
+
+// Purge virtual entries assuming the directory has just been re-read
+//
+// Remove all the entries except:
+//
+// 1) vDirAdd on remotes which can't have empty directories. These will remain
+// virtual as long as the directory is empty. When the directory becomes real
+// (ie files are added) the virtual directory will be removed. This means that
+// directories will disappear when the last file is deleted which is probably
+// OK.
+//
+// 2) vFileAdd that are being written or uploaded
+func (d *Dir) _purgeVirtual() {
+	canHaveEmptyDirectories := d.f.Features().CanHaveEmptyDirectories
+	for name, virtualState := range d.virtual {
+		switch virtualState {
+		case vAddDir:
+			if canHaveEmptyDirectories {
+				// if remote can have empty directories then a
+				// new dir will be read in the listing
+				d._deleteVirtual(name)
+			} else {
+				// leave the empty directory marker
+			}
+		case vAddFile:
+			// Delete all virtual file adds that have finished uploading
+			node, ok := d.items[name]
+			if !ok {
+				// if the object has disappeared somehow then remove the virtual
+				d._deleteVirtual(name)
+				continue
+			}
+			f, ok := node.(*File)
+			if !ok {
+				// if the object isn't a file then remove the virtual as it is wrong
+				d._deleteVirtual(name)
+				continue
+			}
+			if f.writingInProgress() {
+				// if writing in progress then leave virtual
+				continue
+			}
+			if d.vfs.Opt.CacheMode >= vfscommon.CacheModeMinimal && d.vfs.cache.InUse(f.Path()) {
+				// if object in use or dirty then leave virtual
+				continue
+			}
+			d._deleteVirtual(name)
+		default:
+			d._deleteVirtual(name)
+		}
+	}
+}
+
+// Manage the virtuals in a listing
+//
+// This keeps a record of the names listed in this directory so far
+type manageVirtuals map[string]struct{}
+
+// Create a new manageVirtuals and purge the d.virtuals of any entries which can
+// be removed.
+//
+// must be called with the Dir lock held
+func (d *Dir) _newManageVirtuals() manageVirtuals {
+	tv := make(manageVirtuals)
+	d._purgeVirtual()
+	return tv
+}
+
+// This should be called for every entry added to the directory
+//
+// It returns true if this entry should be skipped
+//
+// must be called with the Dir lock held
+func (mv manageVirtuals) add(d *Dir, name string) bool {
+	// Keep a record of all names listed
+	mv[name] = struct{}{}
+	// Remove virtuals if possible
+	switch d.virtual[name] {
+	case vAddFile, vAddDir:
+		// item was added to the dir but since it is found in a
+		// listing is no longer virtual
+		d._deleteVirtual(name)
+	case vDel:
+		// item is deleted from the dir so skip it
+		return true
+	case vOK:
+	}
+	return false
+}
+
+// This should be called after the directory entry is read to update d.items
+// with virtual entries
+//
+// must be called with the Dir lock held
+func (mv manageVirtuals) end(d *Dir) {
+	// delete unused d.items
+	for name := range d.items {
+		if _, ok := mv[name]; !ok {
+			// name was previously in the directory but wasn't found
+			// in the current listing
+			switch d.virtual[name] {
+			case vAddFile, vAddDir:
+				// virtually added so leave virtual item
+			default:
+				// otherwise delete it
+				delete(d.items, name)
+			}
+		}
+	}
+	// delete unused d.virtual~s
+	for name, virtualState := range d.virtual {
+		if _, ok := mv[name]; !ok {
+			// name exists as a virtual but isn't in the current
+			// listing so if it is a virtual delete we can remove it
+			// as it is no longer needed.
+			if virtualState == vDel {
+				d._deleteVirtual(name)
+			}
+		}
+	}
+}
+
 // update d.items and if dirTree is not nil update each dir in the DirTree below this one and
 // set the last read time - must be called with the lock held
 func (d *Dir) _readDirFromEntries(entries fs.DirEntries, dirTree dirtree.DirTree, when time.Time) error {
 	var err error
-	// Cache the items by name
-	found := make(map[string]struct{})
+	mv := d._newManageVirtuals()
 	for _, entry := range entries {
 		name := path.Base(entry.Remote())
 		if name == "." || name == ".." {
 			continue
 		}
 		node := d.items[name]
-		found[name] = struct{}{}
-		virtualState := d.virtual[name]
-		switch virtualState {
-		case vAdd:
-			// item was added to the dir but since it is found in a
-			// listing is no longer virtual
-			delete(d.virtual, name)
-			if len(d.virtual) == 0 {
-				d.virtual = nil
-			}
-			fs.Debugf(d.path, "Removed virtual directory entry %v: %q", virtualState, name)
-		case vDel:
-			// item is deleted from the dir so skip it
+		if mv.add(d, name) {
 			continue
-		case vOK:
 		}
 		switch item := entry.(type) {
 		case fs.Object:
@@ -452,26 +657,7 @@ func (d *Dir) _readDirFromEntries(entries fs.DirEntries, dirTree dirtree.DirTree
 		}
 		d.items[name] = node
 	}
-	// delete unused entries
-	for name := range d.items {
-		if _, ok := found[name]; !ok && d.virtual[name] != vAdd {
-			// item was added to the dir but wasn't found in the
-			// listing - remove it unless it was virtually added
-			delete(d.items, name)
-		}
-	}
-	// delete unused virtuals
-	for name, virtualState := range d.virtual {
-		if _, ok := found[name]; !ok && virtualState == vDel {
-			// We have a virtual delete but the item wasn't found in
-			// the listing so no longer needs a virtual delete.
-			delete(d.virtual, name)
-			fs.Debugf(d.path, "Removed virtual directory entry %v: %q", virtualState, name)
-		}
-	}
-	if len(d.virtual) == 0 {
-		d.virtual = nil
-	}
+	mv.end(d)
 	return nil
 }
 
@@ -528,7 +714,7 @@ func (d *Dir) stat(leaf string) (Node, error) {
 					// duplicate case insensitive match is an error
 					return nil, errors.Errorf("duplicate filename %q detected with --vfs-case-insensitive set", leaf)
 				}
-				// found a case insenstive match
+				// found a case insensitive match
 				ok = true
 				item = node
 			}
@@ -657,6 +843,23 @@ func (d *Dir) Open(flags int) (fd Handle, err error) {
 // Create makes a new file node
 func (d *Dir) Create(name string, flags int) (*File, error) {
 	// fs.Debugf(path, "Dir.Create")
+	// Return existing node if one exists
+	node, err := d.stat(name)
+	switch err {
+	case ENOENT:
+		// not found, carry on
+	case nil:
+		// found so check what it is
+		if node.IsFile() {
+			return node.(*File), err
+		}
+		return nil, EEXIST // EISDIR would be better but we don't have that
+	default:
+		// a different error - report
+		fs.Errorf(d, "Dir.Create stat failed: %v", err)
+		return nil, err
+	}
+	// node doesn't exist so create it
 	if d.vfs.Opt.ReadOnly {
 		return nil, EROFS
 	}
@@ -770,6 +973,7 @@ func (d *Dir) RemoveName(name string) error {
 
 // Rename the file
 func (d *Dir) Rename(oldName, newName string, destDir *Dir) error {
+	// fs.Debugf(d, "BEFORE\n%s", d.dump())
 	if d.vfs.Opt.ReadOnly {
 		return EROFS
 	}
@@ -836,6 +1040,7 @@ func (d *Dir) Rename(oldName, newName string, destDir *Dir) error {
 	destDir.addObject(oldNode)
 
 	// fs.Debugf(newPath, "Dir.Rename renamed from %q", oldPath)
+	// fs.Debugf(d, "AFTER\n%s", d.dump())
 	return nil
 }
 
