@@ -1,8 +1,14 @@
 package vfs
 
+import (
+	"strings"
+)
+
 // Help contains text describing file and directory caching to add to
 // the command help.
-var Help = `
+// Warning: "!" (sic) will be replaced by backticks below,
+//          but the pipe character "|" can be used as is.
+var Help = strings.ReplaceAll(`
 ### VFS - Virtual File System
 
 This command uses the VFS layer. This adapts the cloud storage objects
@@ -19,13 +25,13 @@ about files and directories (but not the data) in memory.
 
 ### VFS Directory Cache
 
-Using the ` + "`--dir-cache-time`" + ` flag, you can control how long a
+Using the !--dir-cache-time! flag, you can control how long a
 directory should be considered up to date and not refreshed from the
 backend. Changes made through the mount will appear immediately or
 invalidate the cache.
 
     --dir-cache-time duration   Time to cache directory entries for. (default 5m0s)
-    --poll-interval duration    Time to wait between polling for changes.
+    --poll-interval duration    Time to wait between polling for changes. Must be smaller than dir-cache-time. Only on supported remotes. Set to 0 to disable. (default 1m0s)
 
 However, changes made directly on the cloud storage by the web
 interface or a different copy of rclone will only be picked up once
@@ -33,7 +39,7 @@ the directory cache expires if the backend configured does not support
 polling for changes. If the backend supports polling, changes will be
 picked up within the polling interval.
 
-You can send a ` + "`SIGHUP`" + ` signal to rclone for it to flush all
+You can send a !SIGHUP! signal to rclone for it to flush all
 directory caches, regardless of how old they are.  Assuming only one
 rclone instance is running, you can reset the cache like this:
 
@@ -50,7 +56,7 @@ Or individual files or directories:
 
 ### VFS File Buffering
 
-The ` + "`--buffer-size`" + ` flag determines the amount of memory,
+The !--buffer-size! flag determines the amount of memory,
 that will be used to buffer data in advance.
 
 Each open file will try to keep the specified amount of data in memory
@@ -63,7 +69,7 @@ yet read. If the buffer is empty, only a small amount of memory will
 be used.
 
 The maximum memory used by rclone for buffering can be up to
-` + "`--buffer-size * open files`" + `.
+!--buffer-size * open files!.
 
 ### VFS File Caching
 
@@ -84,12 +90,12 @@ find that you need one or the other or both.
     --vfs-cache-poll-interval duration   Interval to poll the cache for stale objects. (default 1m0s)
     --vfs-write-back duration            Time to writeback files after last use when using cache. (default 5s)
 
-If run with ` + "`-vv`" + ` rclone will print the location of the file cache.  The
+If run with !-vv! rclone will print the location of the file cache.  The
 files are stored in the user cache file area which is OS dependent but
-can be controlled with ` + "`--cache-dir`" + ` or setting the appropriate
+can be controlled with !--cache-dir! or setting the appropriate
 environment variable.
 
-The cache has 4 different modes selected by ` + "`--vfs-cache-mode`" + `.
+The cache has 4 different modes selected by !--vfs-cache-mode!.
 The higher the cache mode the more compatible rclone becomes at the
 cost of using disk space.
 
@@ -99,11 +105,18 @@ second. If rclone is quit or dies with files that haven't been
 uploaded, these will be uploaded next time rclone is run with the same
 flags.
 
-If using --vfs-cache-max-size note that the cache may exceed this size
+If using !--vfs-cache-max-size! note that the cache may exceed this size
 for two reasons.  Firstly because it is only checked every
---vfs-cache-poll-interval.  Secondly because open files cannot be
+!--vfs-cache-poll-interval!.  Secondly because open files cannot be
 evicted from the cache.
 
+You **should not** run two copies of rclone using the same VFS cache
+with the same or overlapping remotes if using !--vfs-cache-mode > off!.
+This can potentially cause data corruption if you do. You can work
+around this by giving each rclone its own cache hierarchy with
+!--cache-dir!. You don't need to worry about this if the remotes in
+use don't overlap.
+
 #### --vfs-cache-mode off
 
 In this mode (the default) the cache will read directly from the remote and write
@@ -149,7 +162,7 @@ In this mode all reads and writes are buffered to and from disk. When
 data is read from the remote this is buffered to disk as well.
 
 In this mode the files in the cache will be sparse files and rclone
-will keep track of which bits of the files it has dowloaded.
+will keep track of which bits of the files it has downloaded.
 
 So if an application only reads the starts of each file, then rclone
 will only buffer the start of the file. These files will appear to be
@@ -206,6 +219,12 @@ on disk cache file.
     --vfs-read-wait duration   Time to wait for in-sequence read before seeking. (default 20ms)
     --vfs-write-wait duration  Time to wait for in-sequence write before giving error. (default 1s)
 
+When using VFS write caching (--vfs-cache-mode with value writes or full),
+the global flag --transfers can be set to adjust the number of parallel uploads of
+modified files from cache (the related global flag --checkers have no effect on mount).
+
+    --transfers int  Number of file transfers to run in parallel. (default 4)
+
 ### VFS Case Sensitivity
 
 Linux file systems are case-sensitive: two files can differ only
@@ -219,7 +238,7 @@ It is not allowed for two files in the same directory to differ only by case.
 Usually file systems on macOS are case-insensitive. It is possible to make macOS
 file systems case-sensitive but that is not the default
 
-The "--vfs-case-insensitive" mount flag controls how rclone handles these
+The !--vfs-case-insensitive! mount flag controls how rclone handles these
 two cases. If its value is "false", rclone passes file names to the mounted
 file system as-is. If the flag is "true" (or appears without a value on
 command line), rclone may perform a "fixup" as explained below.
@@ -240,4 +259,17 @@ The flag controls whether "fixup" is performed to satisfy the target.
 If the flag is not provided on the command line, then its default value depends
 on the operating system where rclone runs: "true" on Windows and macOS, "false"
 otherwise. If the flag is provided without a value, then it is "true".
-`
+
+### Alternate report of used bytes
+
+Some backends, most notably S3, do not report the amount of bytes used.
+If you need this information to be available when running !df! on the
+filesystem, then pass the flag !--vfs-used-is-size! to rclone.
+With this flag set, instead of relying on the backend to report this
+information, rclone will scan the whole remote similar to !rclone size!
+and compute the total used space itself.
+
+_WARNING._ Contrary to !rclone size!, this flag ignores filters so that the
+result is accurate. However, this is very inefficient and may cost lots of API
+calls resulting in extra charges. Use it as a last resort and only with caching.
+`, "!", "`")
